from . import basic_parser
from bs4 import BeautifulSoup

SECTION_ID = {
    'noun': 'Noun',
    'verb': 'Verb',
    'adverb': 'Adverb',
    'adjective': 'Adjective',
    'preposition': 'Preposition',
    'numeral': 'Numeral',
    'proper_noun': 'Proper_noun',
    'pronoun': 'Pronoun',
    'particle': 'Particle',
    'interjection': 'Interjection',
    'phrase': 'Phrase',
    'contraction': 'Contraction',
    'conjunction': 'Conjunction',
    'article': 'Article',
    'suffix': 'Suffix',
    'abbreviation': 'Abbreviation',
    'initialism': 'Initialism',
    'proverb': 'Proverb',
    'prefix': 'Prefix'
}


class EnglishParser(basic_parser.BasicParser):
    def parse(self):
        if not self.wiktionary_json:
            raise ValueError()
        words = {}
        page_list = list(self.wiktionary_json['query']['pages'].values())
        for page in page_list:
            word = {}
            soup = BeautifulSoup(page['extract'], 'html.parser')
            language_section = ""
            language_tag = soup.find(id='English')
            if language_tag:
                for el in soup.find(id='English').parent.next_siblings:
                    if el.name == 'h2':
                        soup = BeautifulSoup(language_section, 'html.parser')
                        break
                    language_section += str(el)
                word['meanings'] = self.get_meanings(soup)
                words[page['title']] = word
        return words

    def get_meanings(self, soup, meaning_types=list(SECTION_ID.keys()), get_examples=True, get_empty_meaning_types=False):
        meanings = {}
        for meaning_type in meaning_types:
            if meaning_type in SECTION_ID:
                meaning_list = []
                title = soup.find(id=SECTION_ID[meaning_type])
                if title:
                    for li_meaning in title.find_next('ol').find_all('li'):
                        examples = []
                        example_list = li_meaning.find('ul')
                        if example_list:
                            example_list = example_list.extract()
                        if example_list and get_examples:
                            for li_example in example_list.find_all('li'):
                                examples.append(li_example.text)
                        meaning_list.append({
                            'meaning': li_meaning.text.rstrip(),
                            'examples': examples
                        })
                if get_empty_meaning_types or meaning_list:
                    meanings[meaning_type] = meaning_list
        return meanings



import requests

import pywiktionary
import pywiktionary.parsers.basic_parser
import pywiktionary.parsers.english_parser
import pywiktionary.parsers.italian_parser


LANGUAGE_CODES = {
    'italian': 'it',
    'it': 'it',
    'english': 'en',
    'en': 'en'
}

LANGUAGE_PARSERS = {
    'it': pywiktionary.parsers.italian_parser.ItalianParser,
    'en': pywiktionary.parsers.english_parser.EnglishParser
}


class WiktionaryParser:
    def __init__(self, default_language='en', parser_class=None):
        self._base_url = 'https://{lang_code}.wiktionary.org/w/api.php?format=json&action=query&prop=extracts&titles={page_title}'
        self.default_language = LANGUAGE_CODES[default_language] if default_language in LANGUAGE_CODES else 'en'
        if parser_class:
            self.parser_class = parser_class
        else:
            if self.default_language in LANGUAGE_PARSERS:
                self.parser_class = LANGUAGE_PARSERS[self.default_language]
            else:
                self.parser_class = pywiktionary.parsers.basic_parser.BasicParser

    def get_page(self, title, language=None):
        if not language or language not in LANGUAGE_CODES:
            language = self.default_language
        return self.parser_class(requests.get(self._format_url(title, language)).json())

    def _format_url(self, title, language):
        return self._base_url.format(lang_code=language, page_title=title)
